# version 2 is for a set of G,N,T values.
# (G = nof genes, N=nof indivs, T = mutation rate)
#  read.config2() replaces read.config()
#  read.node() is different
# fpath.* functions added


##################################################################################
              
analysis.info.from.config.dir <- function(config.dir) {
network.or.multree <- substr(config.dir, nchar(config.dir), nchar(config.dir))
if (network.or.multree == "n" || network.or.multree == "t") {
  mulTreeVersion <- (network.or.multree == "t")
  scenario.name <- substr(config.dir, nchar(config.dir)-1, nchar(config.dir)-1) 
  stopifnot(scenario.name == "A" || scenario.name == "B" || scenario.name == "C")
  all.files.prefix <- paste(scenario.name, network.or.multree, sep="")
  } else {
  mulTreeVersion <- FALSE
  scenario.name <- substr(config.dir, nchar(config.dir), nchar(config.dir)) 
  stopifnot(scenario.name == "A" || scenario.name == "B" || scenario.name == "C")
  all.files.prefix <- scenario.name
  }

# this should be calculated better!
if (scenario.name == "A") {
  simmul1 <- read.tree(text="((a0:0.02,z0:0.02):0.02,(b0:0.0,z1:0.03):0.01);")  
  simmul2 <- read.tree(text="((a0:0.02,z1:0.02):0.02,(b0:0.0,z0:0.03):0.01);") 
  } else if (scenario.name == "B") {
  simmul1 <- read.tree(text="(((a0:0.02,z0:0.02):0.01,z1:0.03):0.01,b0:0.04);")    
  simmul2 <- read.tree(text="(((a0:0.02,z1:0.02):0.01,z0:0.03):0.01,b0:0.04);")    
  } else if (scenario.name == "C") {
  simmul1 <- read.tree(text="((a0:0.03,(z0:0.02,z1:0.02):0.01):0.01,b0:0.04);")
  simmul2 <- read.tree(text="((a0:0.03,(z1:0.02,z0:0.02):0.01):0.01,b0:0.04);")
  }
list(mulTreeVersion=mulTreeVersion, all.files.prefix=all.files.prefix, simmul1=simmul1, simmul2=simmul2)
}
              
              
               
# file name components - 'type'
fnc.multree <- "1MT"
fnc.simgtree <- "2GT"
fnc.simseq <- "2SQ"
fnc.beastxml <- "3BX"
fnc.sampledgtrees <- "4GTS"
fnc.sampledmultrees <- "4MTS"
fnc.sampledparams <- "4LOG"
fnc.concensusgtree <- "5GTC"
fnc.concensusmultree <- "5MTC"
fnc.DBUGTUNE <- "9DBUGTUNE"
# file name components which appear before config params
fnc.nofgenes <- "g"
fnc.nofindivs <- "n"
fnc.mutrate <- "t"
fnc.replicate <- "r"



fpath.GNTname <- function(config, i) {
  G <- config$configG[i]
  N <- config$configN[i]
  mu <- config$configT[i]
  R <- config$configR[i]
  GNTname <- paste(fnc.nofgenes , G, 
                   fnc.nofindivs, N,                                                                                                                  
                   fnc.mutrate, as.integer(mu*1e9), sep="")
  GNTname                 
  }
  
  
fpath.GNTRname <- function(config, i, r) {
  GNTRname <- paste(fpath.GNTname(config, i), fnc.replicate, r, sep="")
  GNTRname
  }


fpath.GNTdpath <- function(config, i) {
  GNTdpath <- paste(config.dir, "/", all.files.prefix, fpath.GNTname(config, i), sep="")
  GNTdpath
  }
  
  
fpath.multree <- function(config, i) {
  multree.fpath <- paste(fpath.GNTdpath(config, i), "/", all.files.prefix, 
                   fnc.multree, fpath.GNTname(config, i), ".txt", sep="")
  multree.fpath
  }
       
       
fpath.gtrees.fpathbase <- function(config, i, r) {
  gtrees.fpathbase <- paste(fpath.GNTdpath(config, i), "/", all.files.prefix, 
        fnc.simgtree, fpath.GNTRname(config, i, r), "-", sep="")
  gtrees.fpathbase
  }
  
fpath.gtree <- function(config, i, r, g) {
  gtree.fpath <- paste(fpath.gtrees.fpathbase(config, i, r), g, ".txt", sep="")
  gtree.fpath
  }
  
fpath.seqs.fpathbase <- function(config, i, r) {
  seqs.fpathbase <- paste(fpath.GNTdpath(config, i), "/", all.files.prefix, 
        fnc.simseq, fpath.GNTRname(config, i, r), "-", sep="")
  seqs.fpathbase
  }
  
fpath.seq <- function(config, i, r, g) {
  seq.fpath <- paste(fpath.seqs.fpathbase(config, i, r), g, ".txt", sep="")
  seq.fpath
  } 


fpath.beastxml <- function(config, i, r) {
  beastxml <- paste(fpath.GNTdpath(config, i), "/", all.files.prefix, 
            fnc.beastxml, fpath.GNTRname(config, i, r), ".xml", sep="")
  beastxml
  }

fpath.sampledgtrees.fpathbase <- function(config, i, r) {
  sampledgtrees.fpathbase <- paste(fpath.GNTdpath(config, i), "/", all.files.prefix, 
           fnc.sampledgtrees, fpath.GNTRname(config, i, r), "-", sep="")
  sampledgtrees.fpathbase
  }
  
fpath.sampledgtree <- function(config, i, r, g) {
  sampledgtree.fpath <- paste(fpath.sampledgtrees.fpathbase(config, i, r), g, ".txt", sep="")
  sampledgtree.fpath
  }  
  
fpath.sampledmultrees <- function(config, i, r) {
  sampledmultrees.fpath <- paste(fpath.GNTdpath(config, i), "/", all.files.prefix, 
            fnc.sampledmultrees, fpath.GNTRname(config, i, r), ".txt", sep="")
  sampledmultrees.fpath
  }
  
fpath.sampledparams <- function(config, i, r) {  
  sampledparams.fpath <- paste(fpath.GNTdpath(config, i), "/", all.files.prefix,
                 fnc.sampledparams, fpath.GNTRname(config, i, r), ".txt", sep="")
  sampledparams.fpath
  }

fpath.DBUGTUNE <- function(config, i, r) {
  DBUGTUNE.fpath <- paste(fpath.GNTdpath(config, i), "/", all.files.prefix, 
                   fnc.DBUGTUNE, fpath.GNTRname(config, i, r), ".txt", sep="")
  DBUGTUNE.fpath
  }
  

fpath.mccgtrees.fpathbase <- function(config, i, r) {
  mccgtrees.fpathbase <- paste(fpath.GNTdpath(config, i), "/", all.files.prefix, 
           fnc.concensusgtree, fpath.GNTRname(config, i, r), "-", sep="")
  mccgtrees.fpathbase
  }
  
  
fpath.mccgtree <- function(config, i, r, g) {
  mccgtree.fpath <- paste(fpath.mccgtrees.fpathbase(config, i, r), g, ".txt", sep="")
  mccgtree.fpath
  }

fpath.mccmultree <- function(config, i, r) {
  mccmultree.fpath <- paste(fpath.GNTdpath(config, i), "/", all.files.prefix, 
           fnc.concensusmultree, fpath.GNTRname(config, i, r), ".txt", sep="")
  mccmultree.fpath
  }

#################################################################################


parse.progparams <- function(progparam.lines) {
  tokens <- strsplit(progparam.lines, split="[[:blank:]]+")
  
  linenum <- 1
  stopifnot(tokens[[linenum]][1] == "beastseed")
  stopifnot(length(tokens[[linenum]]) == 2)
  beastseed <- as.integer(tokens[[linenum]][2])

  linenum <- 2
  stopifnot(tokens[[linenum]][1] == "beastchainlength")
  stopifnot(length(tokens[[linenum]]) == 2)
  beastchainlength <- as.integer(tokens[[linenum]][2])

  linenum <- 3
  stopifnot(tokens[[linenum]][1] == "beastscreen.logevery")
  stopifnot(length(tokens[[linenum]]) == 2)
  beastscreen.logevery <- as.integer(tokens[[linenum]][2])

  linenum <- 4
  stopifnot(tokens[[linenum]][1] == "beastparams.logevery")
  stopifnot(length(tokens[[linenum]]) == 2)
  beastparams.logevery <- as.integer(tokens[[linenum]][2])

  linenum <- 5
  stopifnot(tokens[[linenum]][1] == "beastgtrees.logevery")
  stopifnot(length(tokens[[linenum]]) == 2)
  beastgtrees.logevery  <- as.integer(tokens[[linenum]][2])

  linenum <- 6
  stopifnot(tokens[[linenum]][1] == "beastmultree.logevery")
  stopifnot(length(tokens[[linenum]]) == 2)
  beastmultree.logevery <- as.integer(tokens[[linenum]][2])

  linenum <- 7
  stopifnot(tokens[[linenum]][1] == "beastdbugtune.logevery")
  stopifnot(length(tokens[[linenum]]) == 2)
  beastdbugtune.logevery  <- as.integer(tokens[[linenum]][2])

  linenum <- 8
  stopifnot(tokens[[linenum]][1] == "treeannburnin")
  stopifnot(length(tokens[[linenum]]) == 2)
  treeannburnin <- as.integer(tokens[[linenum]][2])
  
  list(beastseed=beastseed, beastchainlength=beastchainlength,
       beastscreen.logevery=beastscreen.logevery, beastparams.logevery=beastparams.logevery,
       beastgtrees.logevery=beastgtrees.logevery, beastmultree.logevery=beastmultree.logevery,
       beastdbugtune.logevery=beastdbugtune.logevery, treeannburnin=treeannburnin)
 }



read.config2 <- function(filepath) {
  all.lines <- readLines(filepath)
  ppline <- which(grepl("PROGRAMPARAMETERS", all.lines))
  if (length(ppline) == 0) {
    progparams <- list(beastseed=42, beastchainlength=1500000,
           beastscreen.logevery=10000, beastparams.logevery=1000,
           beastgtrees.logevery=1000, beastmultree.logevery=1000,
           beastdbugtune.logevery=10000, treeannburnin=501)
    config.lines <- all.lines
    } else {
    stopifnot(length(ppline) == 1)
    scenline <- which(grepl("SCENARIO", all.lines)) 
    stopifnot(length(scenline) == 1)
    config.lines <- all.lines[ (scenline+1) : length(all.lines) ]
    progparam.lines <- all.lines[ (ppline+1) : (scenline-1) ]
    progparams <- parse.progparams(progparam.lines)
    }
  
  tokens <- strsplit(config.lines, split="[[:blank:]]+")
  
  linenum <- 1
  stopifnot(tokens[[linenum]][1] == "nofGNRT")
  stopifnot(length(tokens[[linenum]]) == 2)
  nofGNRT <- as.integer(tokens[[linenum]][2])
  
  nofconfigs <- 0
  for (i in 1:nofGNRT) {
    stopifnot(length(tokens[[1+i]]) > 2)
    nofconfigs <- nofconfigs + length(tokens[[linenum]]) - 2
    }
  configG <- rep(0, nofconfigs)
  configN <- rep(0, nofconfigs)
  configR <- rep(0, nofconfigs)
  configT <- rep(0, nofconfigs)
  cfg <- 1
  for (i in 1:nofGNRT) {
    G <- as.integer(tokens[[1+i]][1])
    N <- as.integer(tokens[[1+i]][2])
    R <- as.integer(tokens[[1+i]][3])
    nofTs <- length(tokens[[1+i]]) - 3
    for (j in 1:nofTs) {
      configG[cfg] <- G
      configN[cfg] <- N
      configR[cfg] <- R
      configT[cfg] <- as.double(tokens[[1+i]][3+j])
      cfg <- cfg + 1
      }
    }
  
  linenum <- nofGNRT + 2
  stopifnot(tokens[[linenum]][1] == "noftetratrees")
  stopifnot(length(tokens[[linenum]]) == 2)
  noftetratrees <- as.integer(tokens[[linenum]][2])
  
  linenum <- nofGNRT + 3
  stopifnot(tokens[[linenum]][1] == "genelength")
  stopifnot(length(tokens[[linenum]]) == 2)
  genelength <- as.integer(tokens[[linenum]][2])

  linenum <- nofGNRT + 4
  ditree <- read.ditree.config(tokens, linenum)
  linenum <- ditree$lastlinenum + 1
  
  tetratrees <- read.alltetratrees.config(tokens, linenum, noftetratrees)

  seqs <- list(len=genelength)
  network <- list(noftetratrees=noftetratrees, 
       ditree=ditree, tetratrees=tetratrees)                        
  list(progparams=progparams, configG=configG, configN=configN, configT=configT, configR=configR, 
         seqs=seqs, network=network)
  }
  
  
  
read.ditree.config <- function(tokens, startlinenum) {
  linenum  <- startlinenum
  stopifnot(length(tokens[[linenum]]) == 3)
  stopifnot(tokens[[linenum]][1] == "ditree")
  stopifnot(tokens[[linenum]][2] == "ntips")
  ntips <- as.integer(tokens[[linenum]][3])
  nodes <- as.list(NULL)
  for (i in 1:(2*ntips-1)) {
    linenum <- startlinenum + i
    node <- read.node(tokens, linenum, (i <= ntips), (i == 2*ntips-1))
    nodes <- c(nodes, list(node))
    }
  list(nodes=nodes, lastlinenum=startlinenum+2*ntips-1)
  }
  

  
  
read.alltetratrees.config <- function(tokens, linenum, noftetratrees) {
  tetratrees <- NULL
  for (i in 1:noftetratrees) {
    tettree <- read.tetratree.config(tokens, linenum)
    tetratrees <- c(tetratrees, list(tettree))
    linenum <- tettree$lastlinenum + 1
    }
  tetratrees
  }
  
  
  
read.tetratree.config <- function(tokens, startlinenum) {
  linenum  <- startlinenum
  stopifnot(length(tokens[[linenum]]) == 3)
  stopifnot(tokens[[linenum]][1] == "tetratree")
  stopifnot(tokens[[linenum]][2] == "ntips")
  ntips <- as.integer(tokens[[linenum]][3])
  nodes <- as.list(NULL)
  for (i in 1:(2*ntips-1)) {
    linenum <- startlinenum + i
    node <- read.node(tokens, linenum, (i <= ntips), (i == 2*ntips-1))
    nodes <- c(nodes, list(node))
    }
  linenum <- startlinenum + 2*ntips
  stopifnot(length(tokens[[linenum]]) == 14)
  stopifnot(tokens[[linenum]][1] == "hyb")
  stopifnot(tokens[[linenum]][4] == "join")
  stopifnot(tokens[[linenum]][7] == "foot1")
  stopifnot(tokens[[linenum]][11] == "foot2")
  hybheight <- -1
  hybpop <- -1
  foot1clade <- ""
  foot1height <- -1
  foot1pop <- -1
  joinheight <- -1
  joinpop <- -1
  foot2clade <- ""
  foot2height <- -1
  foot2pop <- -1
  hybheight <- as.double(tokens[[linenum]][2])
  hybpop <- as.double(tokens[[linenum]][3])
  if (tokens[[linenum]][5] != "*") { join = TRUE } else { join = FALSE }
  foot1clade <- paste(tokens[[linenum]][8], "A", sep="")
  foot1height <- as.double(tokens[[linenum]][9])
  foot1pop <- as.double(tokens[[linenum]][10])
  if (join) {
    joinheight <- as.double(tokens[[linenum]][5])
    joinpop <- as.double(tokens[[linenum]][6])
    nodes[[length(nodes)]]$prevpop <- joinpop
    } else {
    foot2clade <- paste(tokens[[linenum]][12], "A", sep="")
    foot2height <- as.double(tokens[[linenum]][13])
    foot2pop <- as.double(tokens[[linenum]][14])
    }
  legs <- list(hybheight=hybheight, hybpop=hybpop,
               foot1clade=foot1clade, foot1height=foot1height, foot1pop=foot1pop,
               joinheight=joinheight, joinpop=joinpop,
               foot2clade=foot2clade, foot2height=foot2height, foot2pop=foot2pop)
  list(nodes=nodes, legs=legs, lastlinenum=startlinenum+2*ntips)
  }
  
  
read.node <- function(tokens, linenum, tip, root) {
  stopifnot(length(tokens[[linenum]]) == 7)
  stopifnot(tokens[[linenum]][4] == "tippop")
  stopifnot(tokens[[linenum]][6] == "prevpop")
  height <- 0
  tippop <- -1
  prevpop <- -1  
  indivs <- -1  
  clade <- tokens[[linenum]][1]
   
  if (root) {
    stopifnot(tokens[[linenum]][7] == "*")
    } else {
    prevpop <- as.double(tokens[[linenum]][7])
    }
  
  if (tip) {
    stopifnot(nchar(clade) == 1)
    } else {
    stopifnot(nchar(clade) >= 5)
    }
  
  stopifnot(tokens[[linenum]][2] == "height")
  height <- as.double(tokens[[linenum]][3])
  stopifnot(tokens[[linenum]][4] == "tippop")
  

  if (tip) {
    stopifnot(height == 0)
    tippop <- as.double(tokens[[linenum]][5])
    } else {
    stopifnot(height > 0)
    }

    
  list(clade=clade, indivs=indivs, height=height,
               tippop=tippop, prevpop=prevpop, root=root)  
  } 






fillin.indivs <- function(network, indivs) {
  for (i in 1:length(network$ditree$nodes)) {
    if (network$ditree$nodes[[i]]$height == 0) {
       network$ditree$nodes[[i]]$indivs <- indivs
      }
    }
  for (j in 1:length(network$tetratrees)) {
    for (i in 1:length(network$tetratrees[[j]]$nodes)) {
      if (network$tetratrees[[j]]$nodes[[i]]$height == 0) {
         network$tetratrees[[j]]$nodes[[i]]$indivs <- indivs
        }
      }
    }
  network
  }
