






make.multree <- function(network.cfg) {
  multree <- as.list(NULL)
  hnodes <- network.cfg$ditree$nodes # 'hnodes' means homoploid nodes
  multree <- homoploidtree2mulsubtree(multree, hnodes, length(hnodes), "A")
  root <- length(multree)
  leglinks <- NULL
  for (i in 1:network.cfg$noftetratrees) {
    hnodes <- network.cfg$tetratrees[[i]]$nodes
    
    legs <- network.cfg$tetratrees[[i]]$legs
    multree <- homoploidtree2mulsubtree(multree, hnodes, length(hnodes), "A")
    lftidx <- length(multree)
    multree[[lftidx]]$hybheight <- legs$hybheight
    multree[[lftidx]]$hybpop <- legs$hybpop
    
    multree <- homoploidtree2mulsubtree(multree, hnodes, length(hnodes), "B")
    rgtidx <- length(multree)
    multree[[rgtidx]]$hybheight <- legs$hybheight
    multree[[rgtidx]]$hybpop <- legs$hybpop
    
    if (legs$foot2clade == "") {
      #  joined legs case
      multree <- c(multree, list(dud.mulnode()))
      joinidx <- length(multree)
      multree[[lftidx]]$anc <- joinidx 
      multree[[joinidx]]$lft <- lftidx
      multree[[rgtidx]]$anc <- joinidx 
      multree[[joinidx]]$rgt <- rgtidx
      multree[[joinidx]]$height <- legs$joinheight      
      llink <- make.leglink(joinidx, legs$foot1clade, legs$foot1height, legs$foot1pop)
      leglinks <- c(leglinks, list(llink))
      } else {
      llink <- make.leglink(lftidx, legs$foot1clade, legs$foot1height, legs$foot1pop)
      leglinks <- c(leglinks, list(llink))
      llink <- make.leglink(rgtidx, legs$foot2clade, legs$foot2height, legs$foot2pop)
      leglinks <- c(leglinks, list(llink))
      }
    }
  footlinkslist <- footlinks.from.leglinks(leglinks)
  multree <- add.legs(multree, footlinkslist)
  multree <- fillin.clades.in.subtree(multree, root)
  multree <- fillin.pops.fromlegs(multree, leglinks)
  multree <- fillin.pops.fromsisters(multree)
  multree
  }                        
  
  


footlinks.from.leglinks <- function(leglinks) {
  footlinkslist <- NULL
  for (i in 1:length(leglinks)) {
    if (!leglinks[[i]]$visited) {
      hips <- NULL
      for (j in 1:length(leglinks)) {
        if (leglinks[[j]]$footclade == leglinks[[i]]$footclade ) {
          hips <- c(hips, list(leglinks[[j]]))
          leglinks[[j]]$visited = TRUE
          }
        }
      footlinkslist <- c(footlinkslist, list(list(hips=hips)))
      }
    }
  footlinkslist
  } 
  

  
  
add.legs <- function(multree, footlinkslist) {
  for (i in 1:length(footlinkslist)) {
    hips <- footlinkslist[[i]]$hips
    fidx <- mulnode.of.clade(multree, hips[[1]]$footclade)
    heights <- rep(0, length(hips))
    for (j in 1:length(hips)) {
      heights[j] <- hips[[j]]$height
      }
    hgtorder <- order(heights)
    for (j in 1:length(hips)) {
      hip <- hips[[ hgtorder[j] ]]
      hidx <- hip$hipidx
      multree <- c(multree, list(dud.mulnode()))
      newidx <- length(multree)
      multree[[newidx]]$anc <- multree[[fidx]]$anc
      multree[[newidx]]$lft <- fidx
      multree[[fidx]]$anc <- newidx
      multree[[newidx]]$rgt <- hidx
      multree[[hidx]]$anc <- newidx
      anc <- multree[[newidx]]$anc
      if (multree[[anc]]$lft == fidx) {
        multree[[anc]]$lft <- newidx
        } else {
        stopifnot(multree[[anc]]$rgt == fidx)
        multree[[anc]]$rgt <- newidx
        }
      multree[[newidx]]$height <- hip$height
      fidx <- newidx
      }           
    }
  multree
  }
  
  
fillin.pops.fromlegs <- function(multree, leglinks) {
  for (i in 1:length(leglinks)) {
    hidx <- leglinks[[i]]$hipidx
    stopifnot(hidx > 0)
    stopifnot(multree[[hidx]]$anc > 0)
    fidx <- multree[[hidx]]$anc
    stopifnot(fidx > 0)
    multree[[fidx]]$rootpop <- leglinks[[i]]$pop
    }
  multree
  }
  
  
fillin.pops.fromsisters <- function(multree) {
  for (i in 1:length(multree)) {
    if (multree[[i]]$anc > 0  &&  multree[[i]]$rootpop < 0) {
      anc <- multree[[i]]$anc 
      if (multree[[anc]]$lft == i) {
        sis <- multree[[anc]]$rgt
        } else {
        stopifnot(multree[[anc]]$rgt == i)
        sis <- multree[[anc]]$lft
        }
      stopifnot(sis > 0)
      stopifnot(multree[[sis]]$rootpop > 0)
      multree[[i]]$rootpop <- multree[[sis]]$rootpop
      }
    }
  multree
  }
  
  
fillin.clades.in.subtree <- function(multree, n) {
  if (multree[[n]]$lft > 0) {
    # if not tip
    multree <- fillin.clades.in.subtree(multree, multree[[n]]$lft)
    multree <- fillin.clades.in.subtree(multree, multree[[n]]$rgt)
    multree[[n]]$clade <- paste(multree[[ multree[[n]]$lft ]]$clade, ",", 
                                multree[[ multree[[n]]$rgt ]]$clade, sep="")
    }
  multree 
  }


  
make.leglink <- function(hipidx, footclade, height, pop) {
  list(hipidx=hipidx, footclade=footclade, height=height, pop=pop, visited=FALSE)
  } 
  
print.multree <- function(multree) {
  cladelen <- 30
  alrlen <- cladelen + 9
  dbllen <- 8
  hlen <- alrlen + 2*(dbllen+1)
  
  header.txt <- paste(rgt.pad("idx   clade", cladelen), "anc  l  r", sep="")
  header.txt <- rgt.pad(header.txt, alrlen)
  header.txt <- paste(header.txt, "     hgt   hybhgt")
  header.txt <- rgt.pad(header.txt, hlen)
  header.txt <- paste(header.txt, "nlin")
  header.txt <- paste(header.txt, "  tippop   hybpop  prevpop")
  cat(header.txt, "\n")
  txt.lens <- list(clade=cladelen, alr=alrlen, hgts=hlen, dbl=dbllen)
  for (i in 1:length(multree)) {
    print.multree.node(i, multree[[i]], txt.lens)
    }
  }  
  



print.multree.node <- function(i, mn, txt.lens) {
  cladelen <- txt.lens$clade      
  alrlen <- txt.lens$alr
  hlen <- txt.lens$hgts
  dbllen <- txt.lens$dbl
  txt <- paste(lft.pad(i,2), mn$clade)
  txt <- rgt.pad(txt, cladelen)
  txt <- paste(txt, lft.pad(mn$anc,2), 
                  lft.pad(mn$lft,2), lft.pad(mn$rgt,2))
  txt <- rgt.pad(txt, alrlen)
  txt <- paste(txt, lft.pad(mn$height, dbllen),  lft.pad(mn$hybheight, dbllen))
  txt <- rgt.pad(txt, hlen)
  txt <- paste(txt, lft.pad(mn$nlins,4))
  txt <- paste(txt, lft.pad(mn$tippop, dbllen), lft.pad(mn$hybpop, dbllen), 
                   lft.pad(mn$rootpop, dbllen))
  cat(txt, "\n")
  }

 
lft.pad <- function(txt, n) {
  while (nchar(txt) < n) {
    txt <- paste("", txt)
    }
  txt
  }

rgt.pad <- function(txt, n) {
  while (nchar(txt) < n) {
    txt <- paste(txt, "")
    }
  txt
  }
 
 
homoploidtree2mulsubtree <- function(multree, hnodes, n, sq) {
  hnode <- hnodes[[n]]
  if (nchar(hnode$clade) == 1) {
    # tip
    newn <- mulnode.from.hnode(hnode, sq) 
    multree <- c(multree, list(newn))
    } else {
    newn <- mulnode.from.hnode(hnode, sq)
    sc <- splitclade(hnode$clade)
    
    hlft <- hnode.of.clade(hnodes, sc$lftclade)
    multree <- homoploidtree2mulsubtree(multree, hnodes, hlft, sq)
    lft <- length(multree)
    
    hrgt <- hnode.of.clade(hnodes, sc$rgtclade)
    multree <- homoploidtree2mulsubtree(multree, hnodes, hrgt, sq)
    rgt <- length(multree)
    
    multree <- c(multree, list(newn))
    here <- length(multree)
    
    multree[[lft]]$anc <- here
    multree[[rgt]]$anc <- here
    multree[[here]]$lft <- lft   
    multree[[here]]$rgt <- rgt  
    }
  multree
  } 
  
  
  
  
mulnode.from.hnode <- function(hnode, sq) {
  x <- strsplit(hnode$clade, split="")[[1]]
  indxs <- grep("[a-z]", x)
  clade <- ""
  n <- 1
  for (i in 1:length(x)) {
    if (n <= length(indxs)  &&  i == indxs[n]) {
      x[i] <- paste(x[i], sq, sep="")
      n <- n+1
      }
    }
  clade <- paste(x, collapse="")
  list(clade=clade, anc=-1, lft=-1, rgt=-1, root=hnode$root,
       height=hnode$height, hybheight=-1, nlins=hnode$indivs, 
       tippop=hnode$tippop, hybpop=-1, rootpop=hnode$prevpop)
}


dud.mulnode <- function() {
list(clade="", anc=-1, lft=-1, rgt=-1, root=FALSE,
       height=0, hybheight=-1, nlins=-1, 
       tippop=-1, hybpop=-1, rootpop=-1)
}


splitclade <- function(clade) {
  x <- strsplit(clade, split="")[[1]]
  stopifnot(x[1]=="(")
  stopifnot(x[length(x)]==")")
  level <- 0
  j <- -1
  for (i in 1:length(x)) {
    if (x[i] == "(") { level <- level + 1 }
    if (x[i] == ")") { level <- level - 1 }
    if (x[i] == ","  &&  level==1) {
      stopifnot(j==-1)
      j <- i
      }
    }
  stopifnot(j>0)
  lftclade <- paste(x[2:(j-1)], collapse="")
  rgtclade <- paste(x[(j+1):(length(x)-1)], collapse="")
  list(lftclade=lftclade, rgtclade=rgtclade) 
  } 


hnode.of.clade <- function(hnodes, clade) {
  j <- -1
  for (i in 1:length(hnodes)) {
    if (hnodes[[i]]$clade == clade) {
      stopifnot(j==-1)
      j <- i
      }
    }
  stopifnot(j>0)
  j
  }		
		
		
mulnode.of.clade <- function(multree, clade) {
  j <- -1
  for (i in 1:length(multree)) {
    if (multree[[i]]$clade == clade) {
      stopifnot(j==-1)
      j <- i
      }
    }
  stopifnot(j>0)
  j
  }
