# 2012-02-16 Reads annotated trees as produced by TreeAnnotator and compares them 
# to lists of topologies for 2 diploids and 3 allotetraploids. 
# It does `strict' and `relaxed' comparisons.

# The case of 2 diploids and 1 allotetraploid is done within simulations: see AlloppSim_1readconfig2.r

# It will fail to read some nexus files containing tree(s)- it is targetted at TreeAnnotator.
# It is inefficient.


library(ape)

setwd("C:/Users/Graham/AAA/Programming/Goteborg/AlloppRcode")
# where this code is

source("AlloppSim_1readconfig2.r")


fpath.for.grouped.multree <- function(results.dir, Sm, G, N, Tt, r) {
paste(results.dir, Sm, "5MTCg", G, "n", N, "t", Tt, "r", r, ".txt", sep="")
}


fpath.for.separate.multree <- function(results.dir, Sm, G, N, Tt, r) {
paste(results.dir, Sm, "g", G, "n", N, "t", Tt, "/", Sm, "5MTCg", G, "n", N, "t", Tt, "r", r, ".txt", sep="")
}


fpath.mccmultree <- function(config, i, r)


remove.annotations <- function(charvect) {
string <- paste(charvect, sep="", collapse="")
noannotbitslist <- strsplit(string, "[[][^][]*[]]")
#print(noannotbitslist)
# [[] matches [
# [^][]* matches anything without ] or [
# []] matches ]
noannotbits <- paste(noannotbitslist[[1]], sep="", collapse="")
#cat("noannotbits\n")
#print(noannotbits)
noannotbits
}   



tree.from.nexus <- function(filepath) {
x <- scan(filepath, what="", sep="\n", quiet=TRUE)
begin.trees.index <- grep("BEGIN TREES;", x=x, ignore.case=TRUE)
all.trees <- x[(begin.trees.index +1):length(x)]
end.trees.index <-  grep("END;|ENDBLOCK;", x=all.trees, ignore.case=TRUE)
all.trees <- all.trees[1:(end.trees.index-1)]
first.tree.index <- grep("TREE", x=all.trees, ignore.case=TRUE)
first.tree <- all.trees[first.tree.index]
translate.block <- all.trees[1:(first.tree.index-1)] 
#print(translate.block)
ntips <- length(translate.block) - 2
stopifnot(grepl("TRANSLATE", translate.block[1], ignore.case=TRUE))
stopifnot(grepl(";", translate.block[ntips+2], ignore.case=TRUE))
translate.block <- translate.block[2:(ntips+1)]
tipnameslist  <-  strsplit(translate.block, "[ \t,]")
#print(tipnameslist)
tipnames <- rep("", ntips)
for (i in 1:ntips) {
  tipnames[i] <- tipnameslist[[i]][length(tipnameslist[[i]])]
  }
tree.charvect <- unlist(strsplit(first.tree, ""))
list(tree.charvect=tree.charvect, tipnames=tipnames)
}


list.correct1tetsubtrees <- function() {
list(zA=c("z1","z0"), zB=c("z0","z1"))
}


list.correct3tetsubtrees <- function() {
xyz0 <- c("(x0,(z0,y0))", "(x0,(z0,y1))", "(x0,(z1,y0))", "(x0,(z1,y1))")
xyz1 <- c("(x1,(z1,y1))", "(x1,(z1,y0))", "(x1,(z0,y1))", "(x1,(z0,y0))")
list(xyzA=c(xyz0,xyz1), xyzB=c(xyz1,xyz0))
}


list.ok3tetsubtrees <- function() {
xyz000 <- c("(x0,(y0,z0))", "(z0,(x0,y0))", "(y0,(z0,x0))")
xyz001 <- c("(x0,(y0,z1))", "(z1,(x0,y0))", "(y0,(z1,x0))")
xyz010 <- c("(x0,(y1,z0))", "(z0,(x0,y1))", "(y1,(z0,x0))")
xyz011 <- c("(x0,(y1,z1))", "(z1,(x0,y1))", "(y1,(z1,x0))")

xyz100 <- c("(x1,(y0,z0))", "(z0,(x1,y0))", "(y0,(z0,x1))")
xyz101 <- c("(x1,(y0,z1))", "(z1,(x1,y0))", "(y0,(z1,x1))")
xyz110 <- c("(x1,(y1,z0))", "(z0,(x1,y1))", "(y1,(z0,x1))")
xyz111 <- c("(x1,(y1,z1))", "(z1,(x1,y1))", "(y1,(z1,x1))")

xyz0 <- rep("", 36)
xyz1 <- rep("", 36)
for (i in 1:3) {
  for (j in 1:3) {
    xyz0[3*(i-1)+j] <- xyz000[i]
    xyz1[3*(i-1)+j] <- xyz111[j]
    xyz0[9 + 3*(i-1)+j] <- xyz001[i]
    xyz1[9 + 3*(i-1)+j] <- xyz110[j]
    xyz0[18 + 3*(i-1)+j] <- xyz010[i]
    xyz1[18 + 3*(i-1)+j] <- xyz101[j]
    xyz0[27 + 3*(i-1)+j] <- xyz011[i]
    xyz1[27 + 3*(i-1)+j] <- xyz100[j]
    }
  }
list(xyzA=c(xyz0,xyz1), xyzB=c(xyz1,xyz0))
}


acceptable.topologies <- function(ABC, ntets, strict) {
if (ABC == "A") {
  part1 <- "((a0,"
  part2 <- "),(b0," 
  part3 <- "));"
  } else if (ABC == "B") {
  part1 <- "(((a0,"
  part2 <- "),"
  part3 <- "),b0);"
  } else if (ABC == "C") {
  part1 <- "((a0,(" 
  part2 <- ","
  part3 <- ")),b0);"
  } else {
  stop("acceptable.topologies() ABC != A, B, or C\n")
  }

if (ntets == 3) {
  if (strict == TRUE) { 
    xyzAB <- list.correct3tetsubtrees()
    } else {
    xyzAB <- list.ok3tetsubtrees()
    }
  xyzA <- xyzAB$xyzA
  xyzB <- xyzAB$xyzB
  n <- length(xyzA)
  stopifnot(n == length(xyzB))
  acceptabletrees <- rep("", n)
  for (i in 1:n) {
    acceptabletrees[i] <- paste(part1, xyzA[i], part2, xyzB[i], part3, sep="")
    }
  } else if (ntets == 1) {
  zAB <- list.correct1tetsubtrees()
  zA <- zAB$zA
  zB <- zAB$zB
  n <- length(zA)
  stopifnot(n == length(zB))
  acceptabletrees <- rep("", n)
  for (i in 1:n) {
    acceptabletrees[i] <- paste(part1, zA[i], part2, zB[i], part3, sep="")
    }
  } else {
  stop("acceptable.topologies() ntets != 1 or 3\n")
  }
acceptabletrees
}  
  
  


fpath.to.phylo <- function(fpath) {
tree.tips.txt <- tree.from.nexus(fpath)
tree.charvect <- tree.tips.txt$tree.charvect
op <- which(tree.charvect=="(")
sc <- which(tree.charvect==";")
stopifnot(length(sc)==1)
charvect <- tree.charvect[op[1]:sc]
tipnames <- tree.tips.txt$tipnames
newick <- paste(charvect, sep="", collapse="")
newick <- remove.annotations(newick)
charvect <- unlist(strsplit(newick, ""))
oporcomma <- which((charvect == "(")  |  (charvect == ","))
colons <- which(charvect == ":")
#print(oporcomma)
#print(colons)
for (i in 1:length(oporcomma)) {
  for (j in 1:length(colons)) {
    if (oporcomma[i]+2 == colons[j]) {    # assume one digit tip index (1...9)
      k <- oporcomma[i] + 1
      isdig <- grepl("[[:digit:]]", charvect[k])[1]
      if (isdig) {
        idx <- as.integer(charvect[k])
        #print(charvect[k])
        charvect[k] <- tipnames[idx]
        }
      }
    }
  }
newick <- paste(charvect, sep="", collapse="")
phylo <- read.tree(text=newick)
}



count.correct.one.replicate <- function(results.dir, Sm, ntets, G, N, Tt, r) {
# grouped for results got from cluster in one dir
#phylo <- fpath.to.phylo(fpath.for.grouped.multree(results.dir, Sm, G, N, Tt, r))
# separate for results in dirs like "Ang3n1t40"
phylo <- fpath.to.phylo(fpath.for.separate.multree(results.dir, Sm, G, N, Tt, r))
correct <- FALSE
ok <- FALSE
if (ntets == 3) {
  if (Sm == "An" || Sm == "At") {
    cts <- acceptable.topologies("A", 3, TRUE)
    okts <- acceptable.topologies("A", 3, FALSE)
    } else if  (Sm == "Bn" || Sm == "Bt") {
    cts <- acceptable.topologies("B", 3, TRUE)
    okts <- acceptable.topologies("B", 3, FALSE)
    } else if  (Sm == "Cn" || Sm == "Ct") {
    cts <- acceptable.topologies("C", 3, TRUE)
    okts <- acceptable.topologies("C", 3, FALSE)
    } else {
    stop("count.correct.one.replicate() Sm != An,At,Bn,Bt,Cn,Ct\n")
    }
  for (i in 1:length(okts)) {
    simmul <- read.tree(text=okts[i])
    sst <- all.equal(simmul, phylo, use.edge.length = FALSE)
    ok <- ok | sst
    }
  } else if  (ntets == 1) {
    cts <- acceptable.topologies("A", 1, TRUE)
    } else if  (Sm == "Bn" || Sm == "Bt") {
    cts <- acceptable.topologies("B", 1, TRUE)
    } else if  (Sm == "Cn" || Sm == "Ct") {
    cts <- acceptable.topologies("C", 1, TRUE)  
  } else {
   stop("count.correct.one.replicate() ntets != 1,3\n")
  }
for (i in 1:length(cts)) {
  simmul <- read.tree(text=cts[i])
  sst <- all.equal(simmul, phylo, use.edge.length = FALSE)
  correct <- correct  |  sst
  }
list(correct=correct, ok=ok)
}


#phylo <- fpath.to.phylo(fpath.for.multree(
#    "C:/AAA/Programming/GoteborgSim/2012-01-20-3tets/J20An/", "An", 3,  1, 80, 20))
#plot.phylo(phylo)
#  1 3       10 11 14 15    17 18 
#  1 3 5 7 9 10 11 14 15 16 17 18 19 20 



collect.results <- function(results.dir, Sm, ntets, ngenes, nindivs, mrates, nreps) {
nG <- length(ngenes)
nN <- length(nindivs)
nT <- length(mrates)
ncorrectresults <- array(0, dim=c(nG, nN, nT))
nokresults <- array(0, dim=c(nG, nN, nT))
for (g in 1:nG) {
  G <- ngenes[g]
  for (n in 1:nN) {
    N <- nindivs[n]
    for (tt in 1:nT) {
      Tt <- mrates[tt]
      oks <- rep(FALSE, nreps)
      corrects <- rep(FALSE, nreps)
      for (r in 1:nreps) {
        ccor <- count.correct.one.replicate(results.dir, Sm, ntets, G, N, Tt, r)
        if (ccor$correct) { 
          corrects[r] <- TRUE
          }
        if (ccor$ok) { 
          oks[r] <- TRUE
          }        
        }
      ncorrect <- length(which(corrects))
      nok <- length(which(oks))
      ncorrectresults[g, n, tt] <- ncorrect
      nokresults[g, n, tt] <- nok
      #cat("G", G, "N", N, "T", Tt, "ncorrect", ncorrect, "nok", nok)
      #cat("\n")
      #cat("correct are:", which(corrects), "ok are:", which(oks), "\n")
      }
    }      
  }
list(ncorrectresults=ncorrectresults, nokresults=nokresults)
}





printresults.latex.3tets <- function(Sm, X3t.ncorrectresults, X3t.nokresults,
                                X3n.ncorrectresults, X3n.nokresults, 
                                ngenes, nindivs, mrates, nreps) {
cat("Results for scenario", Sm, "\n")
cat("Relaxed: topology OK except possibly for tetraploid subtree\n")
nG <- length(ngenes)
nN <- length(nindivs)
nT <- length(mrates)
for (g in 1:nG) {
  G <- ngenes[g]
  for (n in 1:nN) {
    N <- nindivs[n]
    cat(G, ",", N, " ", sep="")
    for (tt in 1:nT) {
      Tt <- mrates[tt]
      cat(" &", X3t.nokresults[g, n, tt], "&", X3n.nokresults[g, n, tt], " ")
      }
    cat("\\\\\n")   
    }      
  }
cat("Strict: topology OK including tetraploid subtree\n")
for (g in 1:nG) {
  G <- ngenes[g]
  for (n in 1:nN) {
    N <- nindivs[n]
    cat(G, ",", N, " ", sep="")
    for (tt in 1:nT) {
      Tt <- mrates[tt]
      cat(" &", X3t.ncorrectresults[g, n, tt], "&", X3n.ncorrectresults[g, n, tt], " ")
      }
    cat("\\\\\n")   
    }      
  }
}



printresults.latex.1tet <- function(Sm, X1t.ncorrectresults, X1n.ncorrectresults,
                                ngenes, nindivs, mrates, nreps) {
cat("Results for scenario with one tet", Sm, "\n")
nG <- length(ngenes)
nN <- length(nindivs)
nT <- length(mrates)
for (g in 1:nG) {
  G <- ngenes[g]
  for (n in 1:nN) {
    N <- nindivs[n]
    cat(G, ",", N, " ", sep="")
    for (tt in 1:nT) {
      Tt <- mrates[tt]
      cat(" &", X1t.ncorrectresults[g, n, tt], "&", X1n.ncorrectresults[g, n, tt], " ")
      }
    cat("\\\\\n")   
    }      
  }
}




collect.print.results.3tets <- function(fpathbase, S, ngenes, nindivs, mrates, nreps) {
n.Sm <- paste(S, "n", sep="")
n.results.dir <- paste(fpathbase, n.Sm, "/", sep="")
n.results <- collect.results(n.results.dir, n.Sm, 3, ngenes, nindivs, mrates, nreps)                  
n.ncorrectresults <- n.results$ncorrectresults
n.nokresults <-  n.results$nokresults

t.Sm <- paste(S, "t", sep="")
t.results.dir <- paste(fpathbase, t.Sm, "/", sep="")
t.results <- collect.results(t.results.dir, t.Sm, 3,
                   ngenes, nindivs, mrates, nreps)                  
t.ncorrectresults <- t.results$ncorrectresults
t.nokresults <-  t.results$nokresults

printresults.latex.3tets(S, t.ncorrectresults, t.nokresults, 
                         n.ncorrectresults, n.nokresults,
                         ngenes, nindivs, mrates, nreps)
}
    
      
collect.print.results.1tet <- function(fpathbase, S, ngenes, nindivs, mrates, nreps) {
n.Sm <- paste(S, "n", sep="")
n.results.dir <- paste(fpathbase, n.Sm, "/", sep="")
n.results <- collect.results(n.results.dir, n.Sm, 1, ngenes, nindivs, mrates, nreps)                  
n.ncorrectresults <- n.results$ncorrectresults

t.Sm <- paste(S, "t", sep="")
t.results.dir <- paste(fpathbase, t.Sm, "/", sep="")
t.results <- collect.results(t.results.dir, t.Sm, 1,
                   ngenes, nindivs, mrates, nreps)                  
t.ncorrectresults <- t.results$ncorrectresults

printresults.latex.1tet(S, t.ncorrectresults, n.ncorrectresults,
                         ngenes, nindivs, mrates, nreps)
}
      

#collect.print.results.3tets("C:/Users/Graham/AAA/Programming/GoteborgSim/2012-01-20-3tets/J20",
#                    "A", ngenes=c(1,3,9), nindivs=c(1,3), mrates=c(40,80), nreps=20)      
      
collect.print.results.1tet("C:/Users/Graham/AAA/Programming/GoteborgSim/2012-02-19-1tet/", "A",
                      ngenes=c(1,3,9), nindivs=c(1,3), mrates=c(40,80), nreps=20)      
 

