# 2012-02-28 For analysing params, esp pop params in simulated data

library(ape)

dpath <- "C:/Users/Graham/AAA/Programming/GoteborgSim/2012-02-19-1tet/"

paramspath <- function(dpath, Sm, G, N, Tt, r) {
paste(dpath, Sm, "/", Sm, "g", G, "n", N, "t", Tt, "/",  Sm, "4LOGg", G, "n", N, "t", Tt, "r", r, ".txt", sep="")
}

burnin <- 500
total <- 1501

collect.etas.SmGNT <- function(Sm, G, N, Tt) {
mletas <- rep(0,20)
for (r in 1:20) {
  ppath <- paramspath(dpath, Sm, G, N, Tt, r)      
  paramsamples <- read.table(ppath, header=TRUE, skip=2)
  log.eta <- log(paramsamples[burnin:total,"species.popMean"])
  med.log.eta <- median(log.eta)
  mletas[r] <- med.log.eta
  }
mletas
}




collect.etas.ST <- function(S, Tt) {
t.etas <- NULL
boxpnames <- character(0)
for (G in c(1,3,9)) {
  for (N in c(1,3)) {
    nam <- paste(G, ",", N, sep="")
    
    MULetas <- collect.etas.SmGNT(paste(S, "t", sep=""), G, N, Tt)
    t.etas <- c(t.etas, list(MULetas))
    MULnam <- paste("MUL", nam)
    boxpnames <- c(boxpnames, MULnam)
    
    NETetas <- collect.etas.SmGNT(paste(S, "n", sep=""), G, N, Tt)
    NETnam <- paste("NET", nam)
    t.etas <- c(t.etas, list(NETetas))
    boxpnames <- c(boxpnames, NETnam)
    }
  }
list(vals=t.etas, nams=boxpnames)
}
 

collect.meanlogpops.SmGNT <- function(Sm, G, N, Tt) {
meanpops <- rep(0,20)
for (r in 1:20) {
  ppath <- paramspath(dpath, Sm, G, N, Tt, r)      
  paramsamples <- read.table(ppath, header=TRUE, skip=2)
  if (substr(Sm,2,2) == "n") {
    branch.meanpops <- rep(0, 8)
    for (pp in 1:length(branch.meanpops)) {
      pop.pp <- paramsamples[burnin:total, paste("apspnetwork.splitPopSize", pp, sep="")]
      branch.meanpops[pp] <-  mean(log(pop.pp))
      }
    } else {
    stopifnot(substr(Sm,2,2) == "t")
    branch.meanpops <- rep(0, 10)
    for (pp in 1:length(branch.meanpops)) {
      pop.pp <- paramsamples[burnin:total, paste("mulSpeciesTree.splitPopSize", pp, sep="")]
      branch.meanpops[pp] <-  mean(log(pop.pp))
      }
    }
  meanpops[r] <- mean(branch.meanpops)
  }
meanpops
}


collect.meanlogpops.ST <- function(S, Tt) {
t.meanlogpops <- NULL
boxpnames <- character(0)
for (G in c(1,3,9)) {
  for (N in c(1,3)) {
    nam <- paste(G, ",", N, sep="")
    
    MULmeanlogpops <- collect.meanlogpops.SmGNT(paste(S, "t", sep=""), G, N, Tt)
    t.meanlogpops <- c(t.meanlogpops, list(MULmeanlogpops))
    MULnam <- paste("MUL", nam)
    boxpnames <- c(boxpnames, MULnam)
    
    NETmeanlogpops <- collect.meanlogpops.SmGNT(paste(S, "n", sep=""), G, N, Tt)
    NETnam <- paste("NET", nam)
    t.meanlogpops <- c(t.meanlogpops, list(NETmeanlogpops))
    boxpnames <- c(boxpnames, NETnam)
    }
  }
list(vals=t.meanlogpops, nams=boxpnames)
}
 

 
plot.etas <- function() {
boxplot(B80etas$vals, range=0, names=B80etas$nams, ylim=c(-10,-2), las=3)
abline(a=log(.004), b=0)
}

plot.meanlogpops <- function() {
boxplot(B80meanlogpops$vals, range=0, names=B80meanlogpops$nams, ylim=c(-10,-2), las=3)
abline(a=log(.008), b=0)
}


#B80etas <- collect.etas.ST("B", 80)
B80meanlogpops <- collect.meanlogpops.ST("B", 80)

#plot.etas()

#setEPS(horizontal = FALSE, onefile = FALSE, paper = "special")
#postscript("C:/Users/Graham/AAA/Docs/PCTeX/Biology/Goteborg/OneHybArticle/Fig4-sim-eta.ps", pointsize=4, paper="special", width=4, height=3)
#plot.etas()
#dev.off()


plot.meanlogpops()

setEPS(horizontal = FALSE, onefile = FALSE, paper = "special")
postscript("C:/Users/Graham/AAA/Docs/PCTeX/Biology/Goteborg/OneHybArticle/Fig4-sim-meanlogpops.ps", pointsize=4, paper="special", width=4, height=3)
plot.meanlogpops()
dev.off()
