# 2012-03-07 Reads annotated trees as produced by TreeAnnotator. 

# It will fail to read some nexus files containing tree(s)- it is targetted at TreeAnnotator.
# It is inefficient.


library(ape)

setwd("C:/Users/Graham/AAA/Programming/Goteborg/AlloppRcode")
# where this code is

source("AlloppSim_1readconfig2.r")

dpath <- "C:/Users/Graham/AAA/Programming/GoteborgSim/2012-02-19-1tet/"




fpath.for.multree <- function(dpath, Sm, G, N, Tt, r) {
paste(dpath, Sm, "/", Sm, "g", G, "n", N, "t", Tt, "/", Sm, "5MTCg", G, "n", N, "t", Tt, "r", r, ".txt", sep="")
}                




remove.annotations <- function(charvect) {
string <- paste(charvect, sep="", collapse="")
noannotbitslist <- strsplit(string, "[[][^][]*[]]")
#print(noannotbitslist)
# [[] matches [
# [^][]* matches anything without ] or [
# []] matches ]
noannotbits <- paste(noannotbitslist[[1]], sep="", collapse="")
#cat("noannotbits\n")
#print(noannotbits)
noannotbits
}   



tree.from.nexus <- function(filepath) {
x <- scan(filepath, what="", sep="\n", quiet=TRUE)
begin.trees.index <- grep("BEGIN TREES;", x=x, ignore.case=TRUE)
all.trees <- x[(begin.trees.index +1):length(x)]
end.trees.index <-  grep("END;|ENDBLOCK;", x=all.trees, ignore.case=TRUE)
all.trees <- all.trees[1:(end.trees.index-1)]
first.tree.index <- grep("TREE", x=all.trees, ignore.case=TRUE)
first.tree <- all.trees[first.tree.index]
translate.block <- all.trees[1:(first.tree.index-1)] 
#print(translate.block)
ntips <- length(translate.block) - 2
stopifnot(grepl("TRANSLATE", translate.block[1], ignore.case=TRUE))
stopifnot(grepl(";", translate.block[ntips+2], ignore.case=TRUE))
translate.block <- translate.block[2:(ntips+1)]
tipnameslist  <-  strsplit(translate.block, "[ \t,]")
#print(tipnameslist)
tipnames <- rep("", ntips)
for (i in 1:ntips) {
  tipnames[i] <- tipnameslist[[i]][length(tipnameslist[[i]])]
  }
tree.charvect <- unlist(strsplit(first.tree, ""))
list(tree.charvect=tree.charvect, tipnames=tipnames)
}



fpath.to.newick <- function(fpath) {
tree.tips.txt <- tree.from.nexus(fpath)
tree.charvect <- tree.tips.txt$tree.charvect
op <- which(tree.charvect=="(")
sc <- which(tree.charvect==";")
stopifnot(length(sc)==1)
charvect <- tree.charvect[op[1]:sc]
tipnames <- tree.tips.txt$tipnames
newick <- paste(charvect, sep="", collapse="")
newick <- remove.annotations(newick)
charvect <- unlist(strsplit(newick, ""))
oporcomma <- which((charvect == "(")  |  (charvect == ","))
colons <- which(charvect == ":")
#print(oporcomma)
#print(colons)
for (i in 1:length(oporcomma)) {
  for (j in 1:length(colons)) {
    if (oporcomma[i]+2 == colons[j]) {    # assume one digit tip index (1...9)
      k <- oporcomma[i] + 1
      isdig <- grepl("[[:digit:]]", charvect[k])[1]
      if (isdig) {
        idx <- as.integer(charvect[k])
        #print(charvect[k])
        charvect[k] <- tipnames[idx]
        }
      }
    }
  }
newick <- paste(charvect, sep="", collapse="")
newick
}



collect.rootheights.SmGNT <- function(Sm, G, N, Tt) {
hgts <- rep(0,20)
for (r in 1:20) {
  mtfpath <- fpath.for.multree(dpath, Sm, G, N, Tt, r)      
  newick <- fpath.to.newick(mtfpath)
  phylo <- read.tree(text=newick)
  #plot.phylo(phylo)
  bts <- branching.times(phylo)
  hgt <- max(bts)
  hgts[r] <- hgt
  }
hgts
}




collect.rootheights.ST <- function(S, Tt) {
t.rootheights <- NULL
boxpnames <- character(0)
for (G in c(1,3,9)) {
  for (N in c(1,3)) {
    nam <- paste(G, ",", N, sep="")
    
    MULrootheights <- collect.rootheights.SmGNT(paste(S, "t", sep=""), G, N, Tt)
    t.rootheights <- c(t.rootheights, list(MULrootheights))
    MULnam <- paste("MUL", nam)
    boxpnames <- c(boxpnames, MULnam)
    
    NETrootheights <- collect.rootheights.SmGNT(paste(S, "n", sep=""), G, N, Tt)
    NETnam <- paste("NET", nam)
    t.rootheights <- c(t.rootheights, list(NETrootheights))
    boxpnames <- c(boxpnames, NETnam)
    }
  }
list(vals=t.rootheights, nams=boxpnames)
}
 

plot.rootheights <- function() {
boxplot(B80$vals, range=0, names=B80$nams, las=3)
abline(a=0.025, b=0)
}



B80 <- collect.rootheights.ST("B", 80)

plot.rootheights()

setEPS(horizontal = FALSE, onefile = FALSE, paper = "special")
postscript("C:/Users/Graham/AAA/Docs/PCTeX/Biology/Goteborg/OneHybArticle/Fig5-sim-rootheights.ps", pointsize=4, paper="special", width=4, height=3)
plot.rootheights()
dev.off()






